from typing import Optional

import click

from anyscale.controllers.jobs_controller import JobController


@click.group(
    "job", hidden=True, help="Interact with highly available jobs within Anyscale."
)
def job_cli() -> None:
    pass


@job_cli.command(
    name="submit", help="Submit a job to run asynchronously on Anyscale.", hidden=True
)
@click.argument("job-config-file", required=True)
@click.option("--name", required=False, default=None, help="Name of job.")
@click.option("--description", required=False, default=None, help="Description of job.")
def submit(
    job_config_file: str, name: Optional[str], description: Optional[str],
) -> None:
    job_controller = JobController()
    job_controller.submit(
        job_config_file, name=name, description=description,
    )


@job_cli.command(
    name="list", help="List information about HA jobs on Anyscale.", hidden=True
)
@click.option("--name", required=False, default=None, help="Filter by job name.")
@click.option("--job-id", required=False, default=None, help="Filter by job id.")
@click.option(
    "--project-id", required=False, default=None, help="Filter by project id."
)
@click.option(
    "--include-all-users",
    is_flag=True,
    default=False,
    help="Include jobs not created by current user.",
)
def list(
    name: Optional[str],
    job_id: Optional[str],
    project_id: Optional[str],
    include_all_users: bool,
) -> None:
    job_controller = JobController()
    job_controller.list(
        name=name,
        job_id=job_id,
        project_id=project_id,
        include_all_users=include_all_users,
    )


@job_cli.command(name="terminate", help="Asynchronously terminate a job.", hidden=True)
@click.option("--id", required=True, help="Id of job.")
def terminate(id: str) -> None:
    # TODO(nikita): Add support for terminating job by name to be consistent with other
    # CLI commands. `--id` is a kwarg because job name will later be an optional arg.
    job_controller = JobController()
    job_controller.terminate(job_id=id)



from typing import Optional

import click

from anyscale.commands.util import OptionPromptNull
from anyscale.controllers.cloud_controller import CloudController


@click.group(
    "cloud",
    short_help="Configure cloud provider authentication for Anyscale.",
    help="""Configure cloud provider authentication and setup
to allow Anyscale to launch instances in your account.""",
)
def cloud_cli() -> None:
    pass


@cloud_cli.command(name="delete", help="Delete a cloud.")
@click.argument("cloud-name", required=False)
@click.option(
    "--cloud-id", help="Cloud id to delete. Alternative to cloud name.", required=False
)
@click.option(
    "--yes", "-y", is_flag=True, default=False, help="Don't ask for confirmation."
)
def cloud_delete(cloud_name: str, cloud_id: str, yes: bool) -> None:
    CloudController().delete_cloud(
        cloud_name=cloud_name, cloud_id=cloud_id, skip_confirmation=yes
    )


@cloud_cli.command(
    name="set-default",
    help=(
        "Sets default cloud for your organization. This operation can only be performed "
        "by organization admins, and the default cloud must have organization level "
        "permissions."
    ),
)
@click.argument("cloud-name", required=False)
@click.option(
    "--cloud-id",
    help="Cloud id to set as default. Alternative to cloud name.",
    required=False,
)
def cloud_set_default(cloud_name: str, cloud_id: str) -> None:
    CloudController().set_default_cloud(cloud_name=cloud_name, cloud_id=cloud_id)


@cloud_cli.command(name="setup", help="Set up a cloud provider.")
@click.option(
    "--bring-cloud-account",
    help="Whether or not this cloud runs in a customer account.",
    required=True,
    prompt="Are you bringing your own cloud provider account to use with Anyscale?",
    type=click.Choice(["yes", "no"], case_sensitive=False),
)
@click.option(
    "--provider",
    help="The cloud provider type.",
    required=True,
    prompt="Provider",
    type=click.Choice(["aws", "gcp"], case_sensitive=False),
)
@click.option(
    "--region",
    cls=OptionPromptNull,
    help="Region to set up the credentials in.",
    required=True,
    prompt="Region",
    default_option="provider",
    default=lambda p: "us-west-2" if p == "aws" else "us-west1",
    show_default=True,
)
@click.option("--name", help="Name of the cloud.", required=True, prompt="Name")
@click.option(
    "--yes", "-y", is_flag=True, default=False, help="Don't ask for confirmation."
)
@click.option(
    "--gcp-folder-id",
    help="Folder ID where the 'Anyscale'  folder will be created.",
    required=False,
    type=int,
)
def setup_cloud(
    bring_cloud_account: str,
    provider: str,
    region: str,
    name: str,
    yes: bool,
    gcp_folder_id: Optional[int],
) -> None:
    CloudController().setup_cloud(
        is_aioa=bring_cloud_account == "no",
        provider=provider,
        region=region,
        name=name,
        yes=yes,
        folder_id=gcp_folder_id,
    )


@cloud_cli.group("config", help="Manage the configuration for a cloud.")
def cloud_config_group() -> None:
    pass


@cloud_config_group.command("update", help="Update the configuration for a cloud.")
@click.argument("cloud-name", required=False)
@click.option(
    "--cloud-id", help="Cloud id to update. Alternative to cloud name.", required=False
)
@click.option(
    "--max-stopped-instances",
    help="Maximum number of stopped instances permitted in the shared instance pool.",
    required=True,
)
def cloud_config_update(
    cloud_name: str, cloud_id: str, max_stopped_instances: int
) -> None:
    CloudController().update_cloud_config(
        cloud_name=cloud_name,
        cloud_id=cloud_id,
        max_stopped_instances=max_stopped_instances,
    )


@cloud_config_group.command("get", help="Get the current configuration for a cloud.")
@click.argument("cloud-name", required=False)
@click.option(
    "--cloud-id", help="Cloud id to delete. Alternative to cloud name.", required=False
)
def cloud_config_get(cloud_name: str, cloud_id: str) -> None:
    print(CloudController().get_cloud_config(cloud_name=cloud_name, cloud_id=cloud_id,))



import json
from typing import Optional
from unittest.mock import Mock, mock_open, patch

import pytest

from anyscale.client.openapi_client import CreateProductionJob, ProductionJobConfig
from anyscale.controllers.job_controller import JobController


def test_submit_job(
    base_mock_api_client: Mock, base_mock_anyscale_api_client: Mock,
) -> None:
    job_controller = JobController(
        api_client=base_mock_api_client,
        anyscale_api_client=base_mock_anyscale_api_client,
    )
    mock_project_definition = Mock()
    mock_project_definition.root = "/some/directory"
    mock_load_project_or_throw = Mock(return_value=mock_project_definition)

    mock_get_project_id = Mock(return_value="mock_project_id")

    mock_config_dict = {
        "entrypoint": "mock_entrypoint",
        "build_id": "mock_build_id",
        "compute_config_id": "mock_compute_config_id",
    }
    with patch(
        "builtins.open", mock_open(read_data=json.dumps(mock_config_dict))
    ), patch.multiple(
        "anyscale.controllers.job_controller",
        load_project_or_throw=mock_load_project_or_throw,
        get_project_id=mock_get_project_id,
    ), patch.multiple(
        "os.path", exists=Mock(return_value=True)
    ):
        job_controller.submit(
            "mock_config_file", name="mock_name", description="mock_description"
        )

    job_controller.api_client.create_job_api_v2_decorated_ha_jobs_create_post.assert_called_once_with(
        CreateProductionJob(
            name="mock_name",
            description="mock_description",
            project_id="mock_project_id",
            config=ProductionJobConfig(**mock_config_dict),
        )
    )


@pytest.mark.parametrize("include_all_users", [False, True])
@pytest.mark.parametrize("name", ["mock_job_name", None])
@pytest.mark.parametrize("job_id", ["mock_job_id", None])
@pytest.mark.parametrize("project_id", ["mock_project_id", None])
def test_list_jobs(
    base_mock_api_client: Mock,
    base_mock_anyscale_api_client: Mock,
    include_all_users: bool,
    name: Optional[str],
    job_id: Optional[str],
    project_id: Optional[str],
) -> None:
    job_controller = JobController(
        api_client=base_mock_api_client,
        anyscale_api_client=base_mock_anyscale_api_client,
    )
    job_controller.api_client.get_user_info_api_v2_userinfo_get = Mock(
        return_value=Mock(result=Mock(id="mock_user_id"))
    )
    job_controller.api_client.list_decorated_jobs_api_v2_decorated_ha_jobs_get = Mock(
        return_value=Mock(results=[Mock(config=Mock(entrypoint=""))])
    )
    job_controller.api_client.get_job_api_v2_decorated_ha_jobs_production_job_id_get = Mock(
        return_value=Mock(result=Mock(config=Mock(entrypoint="")))
    )

    job_controller.list(
        include_all_users=include_all_users,
        name=name,
        job_id=job_id,
        project_id=project_id,
    )

    if job_id:
        job_controller.api_client.get_job_api_v2_decorated_ha_jobs_production_job_id_get.assert_called_once_with(
            job_id
        )
        job_controller.api_client.get_user_info_api_v2_userinfo_get.assert_not_called()
        job_controller.api_client.list_decorated_jobs_api_v2_decorated_ha_jobs_get.assert_not_called()
    else:
        creator_id: Optional[str] = None
        if not include_all_users:
            creator_id = "mock_user_id"
            job_controller.api_client.get_user_info_api_v2_userinfo_get.assert_called_once()
        else:
            job_controller.api_client.get_user_info_api_v2_userinfo_get.assert_not_called()
        job_controller.api_client.get_job_api_v2_decorated_ha_jobs_production_job_id_get.assert_not_called()
        job_controller.api_client.list_decorated_jobs_api_v2_decorated_ha_jobs_get.assert_called_once_with(
            creator_id=creator_id, name=name, project_id=project_id
        )


def test_terminate_job(
    base_mock_api_client: Mock, base_mock_anyscale_api_client: Mock,
) -> None:
    job_controller = JobController(
        api_client=base_mock_api_client,
        anyscale_api_client=base_mock_anyscale_api_client,
    )
    job_controller.terminate("mock_job_id")

    job_controller.api_client.terminate_job_api_v2_decorated_ha_jobs_production_job_id_terminate_post.assert_called_once_with(
        "mock_job_id"
    )



from datetime import datetime
import os
from typing import Optional

import click
import tabulate
import yaml

from anyscale.api import get_anyscale_api_client, get_api_client
from anyscale.cli_logger import _CliLogger
from anyscale.client.openapi_client import CreateProductionJob, ProductionJobConfig
from anyscale.client.openapi_client.api.default_api import DefaultApi
from anyscale.project import get_project_id, load_project_or_throw


class JobController:
    def __init__(
        self,
        api_client: Optional[DefaultApi] = None,
        anyscale_api_client: Optional[DefaultApi] = None,
        log: _CliLogger = _CliLogger(),
    ):
        if api_client is None:
            api_client = get_api_client()
        if anyscale_api_client is None:
            anyscale_api_client = get_anyscale_api_client()
        self.api_client = api_client
        self.anyscale_api_client = anyscale_api_client
        self.log = log

    def submit(
        self, job_config_file: str, name: Optional[str], description: Optional[str]
    ) -> None:
        # TODO(nikita): Remove dependency on project context by providing project_id arg and supporting
        # default projects.
        project_definition = load_project_or_throw()
        project_id = get_project_id(project_definition.root)

        if not os.path.exists(job_config_file):
            raise click.ClickException(f"Config file {job_config_file} not found")

        with open(job_config_file, "r") as f:
            config_dict = yaml.safe_load(f)
        try:
            config_object = ProductionJobConfig(**config_dict)
        except TypeError as e:
            # TODO(nikita): Include more details in error message on what the correct format should be.
            raise click.ClickException(
                f"{e}\n{job_config_file} is not in the correct format for a ProductionJobConfig."
            )

        job = self.api_client.create_job_api_v2_decorated_ha_jobs_create_post(
            CreateProductionJob(
                name=name or "cli-job-{}".format(datetime.now().isoformat()),
                description=description or "Job submitted from CLI",
                project_id=project_id,
                config=config_object,
            )
        ).result

        self.log.info(
            f"Job {job.id} has been successfully submitted. Current state of job: {job.state.current_state}."
        )

    def list(
        self,
        include_all_users: bool,
        name: Optional[str],
        job_id: Optional[str],
        project_id: Optional[str],
    ) -> None:
        jobs_list = []
        if job_id:
            job = self.api_client.get_job_api_v2_decorated_ha_jobs_production_job_id_get(
                job_id
            ).result
            jobs_list.append(job)
        else:
            if not include_all_users:
                creator_id = (
                    self.api_client.get_user_info_api_v2_userinfo_get().result.id
                )
            else:
                creator_id = None
            jobs_resp = self.api_client.list_decorated_jobs_api_v2_decorated_ha_jobs_get(
                project_id=project_id, name=name, creator_id=creator_id
            ).results
            jobs_list.extend(jobs_resp)

        jobs_table = []
        for job in jobs_list:
            jobs_table.append(
                [
                    job.name,
                    job.id,
                    job.cost_dollars,
                    job.project.name,
                    job.state.cluster.name,
                    job.state.current_state,
                    job.creator.username,
                    job.config.entrypoint
                    if len(job.config.entrypoint) < 50
                    else job.config.entrypoint[:50] + " ...",
                ]
            )
        table = tabulate.tabulate(
            jobs_table,
            headers=[
                "NAME",
                "ID",
                "COST",
                "PROJECT NAME",
                "CLUSTER NAME",
                "CURRENT STATE",
                "CREATOR",
                "ENTRYPOINT",
            ],
            tablefmt="plain",
        )
        print(f"Jobs:\n{table}")

    def terminate(self, job_id: str,) -> None:
        job = self.api_client.terminate_job_api_v2_decorated_ha_jobs_production_job_id_terminate_post(
            job_id
        ).result

        self.log.info(f"Job {job.id} has begun terminating...")
        self.log.info(
            f" Current state of job: {job.state.current_state}. Goal state of job: {job.state.goal_state}"
        )



# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class NetworkInterface(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'subnet_id': 'str',
        'groups': 'list[str]',
        'associate_public_ip_address': 'bool'
    }

    attribute_map = {
        'subnet_id': 'SubnetId',
        'groups': 'Groups',
        'associate_public_ip_address': 'AssociatePublicIpAddress'
    }

    def __init__(self, subnet_id=None, groups=None, associate_public_ip_address=True, local_vars_configuration=None):  # noqa: E501
        """NetworkInterface - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._subnet_id = None
        self._groups = None
        self._associate_public_ip_address = None
        self.discriminator = None

        self.subnet_id = subnet_id
        self.groups = groups
        if associate_public_ip_address is not None:
            self.associate_public_ip_address = associate_public_ip_address

    @property
    def subnet_id(self):
        """Gets the subnet_id of this NetworkInterface.  # noqa: E501


        :return: The subnet_id of this NetworkInterface.  # noqa: E501
        :rtype: str
        """
        return self._subnet_id

    @subnet_id.setter
    def subnet_id(self, subnet_id):
        """Sets the subnet_id of this NetworkInterface.


        :param subnet_id: The subnet_id of this NetworkInterface.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and subnet_id is None:  # noqa: E501
            raise ValueError("Invalid value for `subnet_id`, must not be `None`")  # noqa: E501

        self._subnet_id = subnet_id

    @property
    def groups(self):
        """Gets the groups of this NetworkInterface.  # noqa: E501


        :return: The groups of this NetworkInterface.  # noqa: E501
        :rtype: list[str]
        """
        return self._groups

    @groups.setter
    def groups(self, groups):
        """Sets the groups of this NetworkInterface.


        :param groups: The groups of this NetworkInterface.  # noqa: E501
        :type: list[str]
        """
        if self.local_vars_configuration.client_side_validation and groups is None:  # noqa: E501
            raise ValueError("Invalid value for `groups`, must not be `None`")  # noqa: E501

        self._groups = groups

    @property
    def associate_public_ip_address(self):
        """Gets the associate_public_ip_address of this NetworkInterface.  # noqa: E501


        :return: The associate_public_ip_address of this NetworkInterface.  # noqa: E501
        :rtype: bool
        """
        return self._associate_public_ip_address

    @associate_public_ip_address.setter
    def associate_public_ip_address(self, associate_public_ip_address):
        """Sets the associate_public_ip_address of this NetworkInterface.


        :param associate_public_ip_address: The associate_public_ip_address of this NetworkInterface.  # noqa: E501
        :type: bool
        """

        self._associate_public_ip_address = associate_public_ip_address

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, NetworkInterface):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, NetworkInterface):
            return True

        return self.to_dict() != other.to_dict()
